﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NbuLibrary.Core.DataModel
{
    public class EntityRulesCollection : ICollection<EntityRuleModel>
    {
        private List<EntityRuleModel> _data;

        public EntityRulesCollection()
        {
            _data = new List<EntityRuleModel>();
        }

        public IEnumerable<EntityRuleModel> GetRulesFor(PropertyModel property)
        {
            return _data.Where(r =>
            {
                if (r is RequiredRuleModel)
                    return (r as RequiredRuleModel).Property.Name == property.Name;
                else if (r is UniqueRuleModel)
                    return (r as UniqueRuleModel).Properties.Contains(property);
                else
                    return false;
            });
        }


        public void Add(EntityRuleModel item)
        {
            _data.Add(item);
        }

        public void Clear()
        {
            _data.Clear();
        }

        public bool Contains(EntityRuleModel item)
        {
            if (item is UniqueRuleModel)
            {
                var ur = item as UniqueRuleModel;
                var urProps = ur.Properties.Select(p => p.Name).ToList();
                return _data.Find(r =>
                {
                    if (r is UniqueRuleModel == false) return false;
                    var ur2 = r as UniqueRuleModel;
                    if (urProps.Count != ur2.Properties.Count()) return false;
                    var ur2Props = ur2.Properties.Select(p => p.Name).ToList();
                    foreach (var p in urProps)
                        if (!ur2Props.Contains(p, StringComparer.InvariantCultureIgnoreCase))
                            return false;

                    return true;
                }) != null;
            }
            else if (item is RequiredRuleModel)
            {
                var rr = item as RequiredRuleModel;
                return _data.Find(ex =>
                {
                    var rr2 = ex as RequiredRuleModel;
                    return rr2 != null && rr2.Property.Name.Equals(rr.Property.Name, StringComparison.InvariantCultureIgnoreCase);
                }) != null;
            }
            else return _data.Contains(item);
        }

        public void CopyTo(EntityRuleModel[] array, int arrayIndex)
        {
            _data.CopyTo(array, arrayIndex);
        }

        public int Count
        {
            get { return _data.Count; }
        }

        public bool IsReadOnly
        {
            get { return false; }
        }

        public bool Remove(EntityRuleModel item)
        {
            return _data.Remove(item);
        }

        public IEnumerator<EntityRuleModel> GetEnumerator()
        {
            return _data.GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return _data.GetEnumerator();
        }
    }
}
