﻿using NbuLibrary.Core.Domain;
using NbuLibrary.Core.Services;
using NbuLibrary.Core.Services.tmp;
using NbuLibrary.Web.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;

namespace NbuLibrary.Web.Controllers
{
    [AllowAnonymous]
    public class LoginController : Controller
    {
        private ISecurityService _securityService;
        private IEntityOperationService _entityService;

        public LoginController(ISecurityService securityService, IEntityOperationService entityService)
        {
            _securityService = securityService;
            _entityService = entityService;
        }
        //
        // GET: /Login/

        public ActionResult Index(string ReturnUrl)
        {
            return View(new LoginViewModel() { ReturnUrl = ReturnUrl });
        }

        [HttpPost]
        public ActionResult Index(LoginViewModel model)
        {
            if (ModelState.IsValid)
            {
                var result = _securityService.Login(model.Email, model.Password, model.RememberMe);
                if (result == LoginResult.Success)
                {
                    if (string.IsNullOrEmpty(model.ReturnUrl))
                        return Redirect("/");
                    else
                        return Redirect(model.ReturnUrl);
                }
                else
                {
                    string error = null;
                    if (result == LoginResult.InvalidCredentials)
                        error = "Username or password not valid!";
                    else
                        error = "Your user registration is not yet authorized by administrator. Please, contact the administrator to check the status of your registration request.";

                    ModelState.AddModelError(string.Empty, error);
                    return View(model);
                }
            }
            else
                return View(model);
        }

        [HttpGet]
        public ActionResult SignOut()
        {
            _securityService.Logout();
            return View();
        }

        [HttpGet]
        public ActionResult Register()
        {
            var model = new RegisterGroupModel();
            var query = new EntityQuery2(UserGroup.ENTITY);
            query.AllProperties = true;
            query.WhereIs("UserType", UserTypes.Customer);
            using (_securityService.BeginSystemContext())
            {
                model.AvailableGroups = _entityService.Query(query).Select(e => new UserGroup(e));
            }
            return View(model);
        }

        [HttpPost]
        public ActionResult Register(RegisterGroupModel model)
        {
            using (_securityService.BeginSystemContext())
            {
                var query = new EntityQuery2(UserGroup.ENTITY);
                query.AllProperties = true;
                query.WhereIs("UserType", UserTypes.Customer);
                model.AvailableGroups = _entityService.Query(query).Select(e => new UserGroup(e));
            }

            if (!ModelState.IsValid)
            {
                return View(model);
            }
            else
            {
                var selectedGroup = model.AvailableGroups.Single(g => g.Id == model.UserGroup);
                if (selectedGroup.Name.Equals("Студенти", StringComparison.InvariantCultureIgnoreCase)
                    || selectedGroup.Name.Equals("Преподаватели", StringComparison.InvariantCultureIgnoreCase))
                {
                    return View("RegisterStudent", new RegisterStudentViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
                }
                else if (selectedGroup.Name.Equals("Външни", StringComparison.InvariantCultureIgnoreCase))
                    return View("RegisterExternal", new RegisterExternalViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
                else
                    return View("RegisterOther", new RegisterViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
            }
        }

        [HttpPost]
        public ActionResult RegisterStudent(RegisterStudentViewModel model)
        {
            if (!ModelState.IsValid)
            {
                return View(model);
            }
            else
                return FinishResigration(new RegisterViewModel(model));
        }

        [HttpPost]
        public ActionResult RegisterExternal(RegisterExternalViewModel model)
        {
            if (!ModelState.IsValid)
            {
                return View(model);
            }
            else
                return FinishResigration(new RegisterViewModel(model));
        }

        [HttpPost]
        public ActionResult RegisterOther(RegisterViewModel model)
        {
            if (!ModelState.IsValid)
            {
                return View(model);
            }
            else
                return FinishResigration(model);
        }

        [HttpPost]
        public ActionResult FinishResigration(RegisterViewModel model)
        {
            User user = new User()
            {
                Email = model.Email,
                Password = model.Password,
                FirstName = model.FirstName,
                MiddleName = model.MiddleName,
                LastName = model.LastName,
                FacultyNumber = model.FacultyNumber,
                CardNumber = model.CardNumber,
                PhoneNumber = model.CardNumber,
                UserType = UserTypes.Customer
            };
            var update = new EntityUpdate(user);
            update.Attach(UserGroup.ENTITY, UserGroup.DEFAULT_ROLE, model.UserGroup);
            EntityOperationResult result = null;
            using (_securityService.BeginSystemContext())
            {
                result = _entityService.Update(update);
            }

            if (result.Success)
                return View("RegisterComplete");
            else
            {
                IEnumerable<UserGroup> availableGroups = null;
                using (_securityService.BeginSystemContext())
                {
                    var query = new EntityQuery2(UserGroup.ENTITY);
                    query.AllProperties = true;
                    query.WhereIs("UserType", UserTypes.Customer);
                    availableGroups = _entityService.Query(query).Select(e => new UserGroup(e));
                }

                if (result.Errors == null || result.Errors.Count == 0)
                    ModelState.AddModelError("", "Unexpected error occured. Please, try again. If there is still a problem, contact the administrator.");
                else
                {
                    foreach (var err in result.Errors)
                    {
                        ModelState.AddModelError("", err.Message);
                    }
                }

                var selectedGroup = availableGroups.Single(g => g.Id == model.UserGroup);
                if (selectedGroup.Name.Equals("Студенти", StringComparison.InvariantCultureIgnoreCase)
                    || selectedGroup.Name.Equals("Преподаватели", StringComparison.InvariantCultureIgnoreCase))
                {
                    return View("RegisterStudent", new RegisterStudentViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
                }
                else if (selectedGroup.Name.Equals("Външни", StringComparison.InvariantCultureIgnoreCase))
                    return View("RegisterExternal", new RegisterExternalViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
                else
                    return View("RegisterOther", new RegisterViewModel() { UserGroup = model.UserGroup, UserGroupName = selectedGroup.Name });
            }
        }
    }
}
