﻿/// <reference path="jquery-1.8.3.js" />
/// <reference path="../knockout-2.2.0.debug.js" />


//--------------------

//TODO: Move from here
//--------TMP----------------

NbuLib.QueryBuilder = {
    makeFilter: function (prop, op, vals) {
        return { Property: prop, Operator: op, Values: vals };
    },

    makeRelFilter: function (entity, role, prop, op, vals) {
        return {
            EntityName: entity,
            Role: role,
            PropertyRules: [{ Property: prop, Operator: op, Values: vals }]
        };
    },

    makeSort: function (prop, desc, role) {
        return { Property: prop, Descending: desc || false, Role: role };
    }
}

//---------------------------

NbuLib.Core.ViewStore = function () {
    this._hash = {};
}

NbuLib.Core.ViewStore.prototype.getView = function (view, callback) {
    var self = this;

    if (this._hash[view]) {
        if (callback)
            callback(this._hash[view]);
        else
            return this._hash[view];
    }
    else {
        $.ajax({
            type: 'GET',
            url: '/api/ViewDef/Get',
            data: { viewName: view },
            async: !!callback,
            success: function (def) {
                self._hash[view] = def;
                if (callback)
                    callback(def);
            }
        });

        if (!callback)
            return this._hash[view];
    }
}

NbuLib.Core.ViewStore.prototype.saveView = function (view, callback) {
    var self = this;

    if (this._hash[view])
        this._hash[view] = null;


    var response = null;
    $.ajax({
        type: 'POST',
        url: '/api/ViewDef/Save',
        data: { Raw: JSON.stringify(view) },
        async: !!callback,
        success: function (res) {
            response = res;
            if (callback)
                callback(res);
        }
    });

    if (!callback)
        return response;
}

NbuLib.Core.EntityStore = function () {
    return {
        getDefinition: function (entity, callback) {
            var result = null;
            $.ajax({
                type: 'POST',
                url: '/api/Entity/GetDefinition?entity=' + entity,
                async: !!callback,
                success: function (res) {
                    if (!!callback)
                        callback(res);
                    else
                        result = res;
                }
            });
            if (!callback)
                return result;
        },
        getEnum: function (enumClass, callback) {
            var syncResult = undefined;
            $.ajax({
                type: 'GET',
                url: '/api/Entity/GetEnum?enumClass=' + enumClass,
                async: !!callback,
                success: function (res) {
                    if (callback)
                        callback(res);
                    else syncResult = res;
                }
            });

            return syncResult;
        }
    };
}


NbuLib.Core.Dialog = function (dom) {
    var _modal = $(dom);
    return {
        close: function () {
            _modal.modal('hide');
        },
        showProgress: function () {
            var btn = _modal.find('.modal-footer .save').hide();
            $('<img class="save-progress" src="/Content/ajax-loader-btn.gif" alt="Sending..."/>').insertAfter(btn);
        },
        hideProgress: function () {
            _modal.find('.save-progress').remove();
            _modal.find('.modal-footer .save').show();
        }
    }
}

NbuLib.Core.Client = function () {

    this.Menu = new MenuViewModel();
    this.Modules = ko.observableArray([]);
    this._permissions = [];
    this._availModules = {};
    this.registrationQueue = [];

    this._loaded = false;

    this.ViewStore = new NbuLib.Core.ViewStore();
    this._hooks = {
        MenuInit: []
    };
}

NbuLib.Core.Client.prototype.hookMenuInit = function (hook) {
    this._hooks.MenuInit.push(hook);
}

NbuLib.Core.Client.prototype.initMenu = function () {
    var self = this;
    var menu = $('#mainMenu');

    this.Menu.AddItem(new MenuViewModel({ Name: 'Home', Order: 1, Action: "/" }));
    //this.Menu.AddItem(new MenuViewModel({ Name: 'Views', Order: 2, Action: "/ViewMgr/" }));
    //this.Menu.AddItem(new MenuViewModel({
    //    Name: 'Install', Order: 1, Action: function () {
    //        $('#viewport').html('');
    //        $('<iframe src="/Home/Install" />').appendTo('#viewport').css('width:100%;height:500px;');
    //    }
    //}));

    this.Menu.AddItem(new MenuViewModel({
        Name: 'Profile',
        Items: [
            new MenuViewModel({ Name: 'Sign Out', Order: 9999, Action: '/Login/SignOut' })
        ], Order: 9999
    }));

    ko.applyBindings(this.Menu, menu[0]);


    $(this._hooks.MenuInit).each(function () {
        this.call(window, self.Menu);
    });

    //var idx = window.location.search.indexOf('openItem') + 11;
    //var len = window.location.search.indexOf('&', idx) - idx;
    //console.log(window.location.search.substr(idx, len));
    //var parts = window.location.search.substr(idx, len).split("|");
    //this.Menu.open(parts);
}

NbuLib.Core.Client.prototype.setPermissions = function (perms) {
    var self = this;

    $(perms).each(function () {
        self._permissions.push(this);
        self._availModules[this.ModuleID] = true;
    });

    self._loaded = true;
}

NbuLib.Core.Client.prototype.hasPermission = function (module) {
    return !!this._availModules[module];
}

NbuLib.Core.Client.prototype.register = function (module) {
    //this.Modules.push(module);
    //this._moduleHash[module.Name] = module;

    //if (this._loaded) {
    //    //render menu
    //    if (!client._availModules[module.Id])
    //        return;

    //    var li = $('<li class="dropdown"/>').insertBefore($("#mainMenu").children('li').last());
    //    $('<a class="dropdown-toggle" data-toggle="dropdown" href="#" role="button">' + module.Name + ' <b class="caret"></b></a>').appendTo(li);
    //    var ul = $('<ul class="dropdown-menu" role="menu" aria-labelledby="dLabel"/>').appendTo(li);
    //    $(module.Pages).each(function () {
    //        var page = this.Name;
    //        var li = $('<li />').appendTo(ul);
    //        var a = $('<a href="#">' + this.Name + '</a>')
    //            .appendTo(li)
    //            .click(function (e) {
    //                e.preventDefault();
    //                client.openPage(module.Name, page);
    //                $("#mainMenu").find('li.active a').text(page);
    //            });
    //    });
    //}
    //else {
    //    this.registrationQueue.push(module);
    //}
}

NbuLib.Core.Client.prototype.showLoader = function (viewModel) {
    //$('#loader').show().center($('#viewport'));
    $('#loader').show().center();
    var subsciption = viewModel.Loaded.subscribe(function (newValue) {
        //console.log(newValue);
        if (newValue) {
            subsciption.dispose();
            $('#loader').fadeOut();
        }
    });
}

NbuLib.Core.Client.prototype.openPage = function (moduleName, pageName) {
    var viewport = $('#viewport').get(0);
    var currentVM = ko.dataFor(viewport);
    if (currentVM && currentVM.dispose)
        currentVM.dispose();
    ko.cleanNode(viewport);
    var page = this._moduleHash[moduleName].getPage(pageName);
    page.open(viewport);
    if (!page.ViewModelInst.Loaded())
        this.showLoader(page.ViewModelInst);
}

//TODO: refactor using class, not instance of a VM
NbuLib.Core.Client.prototype.showInViewport = function (vm, opt) {

    //TODO: Common code!
    var viewport = $('#viewport').get(0);
    var currentVM = ko.dataFor(viewport);
    if (currentVM && currentVM.dispose)
        currentVM.dispose();
    ko.cleanNode(viewport);
    //-----------------------------------------

    $(viewport).attr("data-bind", "template : {name : '" + opt.templ + "'}");
    ko.applyBindings(vm, viewport);
}


NbuLib.Core.Client.prototype.showInDialog = function (vm, opt) {
    var modal = $('#modalWindow').clone().appendTo(document.body);

    opt.width = opt.width || 560;
    modal.width(opt.width);

    modal.css('margin-left', -(opt.width / 2));

    modal.find('#modalTitle').text(opt.title);
    var modalBody = modal.find('.modal-body');
    modalBody.attr('data-bind', "template : {name : '" + opt.templ + "'}");
    ko.applyBindings(vm, modalBody[0]);

    if (opt.act) {
        modal.find('.modal-footer .save')
            .show()
            .text(opt.act.title)
            .bind('click', function (e) {
                e.preventDefault();

                var form = modalBody.find('form');
                if (form.length > 0 && !opt.noValidate) {
                    if (!form.valid())
                        return;
                }
                opt.act.callback(vm);
            });
    }
    else
        modal.find('.modal-footer .save').hide();

    modal.modal('show');
    modal.bind('hidden', function () {
        ko.cleanNode(modal.find('.modal-body')[0]);
        modal.remove();
    });

    return new NbuLib.Core.Dialog(modal);
}

NbuLib.Core.Client.prototype.closeDialog = function () {
    var modal = $('.modal').last();
    modal.modal('hide');
}

NbuLib.Core.Client.prototype.getActiveDialog = function () {
    return $('.modal').last();
}

NbuLib.Core.Client.prototype.confirmDialog = function (txt, callback) {

    var modal = $('#modalWindow').clone().appendTo(document.body);

    modal.find('#modalTitle').html("&nbsp;");
    var modalBody = modal.find('.modal-body');
    modalBody.html(txt);

    modal.find('.modal-footer .save')
        .show()
        .text("Yes")
        .bind('click', function (e) {
            e.preventDefault();
            callback(true);
        });

    modal.find('.modal-footer .cancel')
        .text('No')
        .bind('click', function (e) {
            e.preventDefault();
            callback(false);
        });

    modal.modal('show');
    modal.bind('hidden', function () {
        modal.remove();
    });
}

NbuLib.Core.Client.prototype.showSuccess = function (html) {
    noty({ text: html, type: 'success', layout: 'bottom' });
}

NbuLib.Core.Client.prototype.showError = function (html) {
    noty({ text: html, type: 'error', layout: 'bottom' });
}

NbuLib.Core.Client.prototype.getCurrentUser = function () {
    var self = this;
    if (!self._user)
        $.ajax(
            {
                url: '/Home/GetCurrentUser',
                async: false,
                success: function (resp) {
                    self._user = resp;
                }
            });

    return self._user;
}

NbuLib.Core.Client.prototype.get = function (id, entityName, includes, callback) {

    var request = {
        Query: {
            EntityName: entityName,
            PropertyRules: [NbuLib.QueryBuilder.makeFilter("Id", "is", [id])]
        },
        Includes: includes
    };

    $.ajax({
        type: 'POST',
        url: '/api/Entity/Get',
        data: request,
        dataType: 'json',
        success: function (entity) {
            console.log(entity);
            if (callback)
                callback(entity);
        },
        error: this.errorHandler
    });
}

NbuLib.Core.Client.prototype.search = function (params, callback) {

    var request = {
        Query: {
            EntityName: params.Entity,
            PropertyRules: params.Filter || [],
            RelationRules: params.RelFilter || []
        },
        SortBy: params.SortBy || [],
        Includes: params.Includes || []
    };

    $.ajax({
        type: 'POST',
        url: '/api/Entity/Search',
        data: request,
        dataType: 'json',
        success: function (entities) {
            console.log(entities);
            if (callback)
                callback(entities);
        }
    });
}

NbuLib.Core.Client.prototype.update = function (update, callback) {

    var request = {
        EntityName: update.Entity,
        PropertyUpdates: [],
        RelationUpdates: []
    };

    $(update.Rels).each(function () {
        var rel = { EntityName: this.Entity, Role: this.Role, PropertyUpdates: [], Operation: this.Op };
        $.each(this.Props || [], function (k, v) {
            rel.PropertyUpdates.push({ Name: k, Value: v });
        });
        rel.PropertyUpdates.push({ Name: 'Id', Value: this.Id });
        if (this.Id)
            request.RelationUpdates.push(rel);
    });

    $.each(update.Props || [], function (k, v) {
        request.PropertyUpdates.push({ Name: k, Value: v });
    });

    request.PropertyUpdates.push({ Name: 'Id', Value: update.Id });

    console.log(request);
    $.ajax({
        type: 'POST',
        url: '/api/Entity/Update',
        data: request,
        dataType: 'json',
        success: function (response) {
            console.log(response);
            if (callback)
                callback(response);
        },
        error: this.errorHandler
    });
}

NbuLib.Core.Client.prototype.errorHandler = function (response, status) {
    var error = $.parseJSON(response.responseText);
    noty({ text: "<b>" + error.Message + "</b><p>" + error.ExceptionMessage, type: 'error', layout: 'bottom' });
}

NbuLib.Core.Client.prototype.deleteEntity = function (entity, id, callback) {
    $.ajax({
        type: 'POST',
        url: '/api/Entity/Delete',
        data: { EntityName: entity, Id: id },
        dataType: 'json',
        success: function (response) {
            console.log(response);
            if (callback)
                callback(response);
        },
        error: this.errorHandler
    });
}

var client = new NbuLib.Core.Client();

$(function () {
    $('#loader').hide();

    $.get('/Home/LoadPermissions', { ts: new Date().getTime() }, function (perms) {

        client.setPermissions(perms);

        //init Menu
        client.initMenu();



        //var menu = $('#mainMenu');

        //$(client.registrationQueue).each(function () {
        //    var module = this;

        //    if (!client._availModules[module.Id])
        //        return;

        //    var moduleName = module.Name;
        //    var li = $('<li class="dropdown"/>').appendTo(menu);
        //    $('<a class="dropdown-toggle" data-toggle="dropdown" href="#" role="button">' + moduleName + ' <b class="caret"></b></a>').appendTo(li);
        //    var ul = $('<ul class="dropdown-menu" role="menu" aria-labelledby="dLabel"/>').appendTo(li);
        //    $(module.Pages).each(function () {
        //        var page = this.Name;
        //        var li = $('<li />').appendTo(ul);
        //        var a = $('<a href="#">' + this.Name + '</a>')
        //            .appendTo(li)
        //            .click(function (e) {
        //                e.preventDefault();
        //                client.openPage(moduleName, page);
        //                menu.find('li.active a').text(page);
        //            });
        //    });
        //});
        //while (client.registrationQueue.length)
        //    client.registrationQueue.pop();


        //var profileLink = $('<li class="dropdown"/>').appendTo(menu);
        //$('<a class="dropdown-toggle" data-toggle="dropdown" href="#" role="button"> <i class="icon icon-user"></i> Profile <b class="caret"></b></a>').appendTo(profileLink);
        //var profileSubmenu = $('<ul class="dropdown-menu" role="menu" aria-labelledby="dLabel"/>').appendTo(profileLink);
        //var logOutLink = $('<li><a href="/Login/SignOut">Sign Out</a></li>').appendTo(profileSubmenu);

        //$('.dropdown-trigger').dropdown();
    });
});

function MenuViewModel(opts) {
    console.log(opts);

    opts = opts || {};

    var _name = opts.Name;
    var _order = opts.Order || 10;
    var _menuItems = ko.observableArray(opts.Items || []);
    var _action = opts.Action;

    return {
        Name: _name,
        Order: _order,
        AddItem: function (item) {
            _menuItems.push(item);
        },
        AddItemToMenu: function (menuName, item) {
            ko.utils.arrayForEach(_menuItems(), function (it) {
                if (it.Name == menuName)
                    it.AddItem(item);
            });
        },
        MenuItems: ko.dependentObservable(function () {
            return _menuItems.slice().sort(function (a, b) { return a.Order < b.Order ? -1 : 1 });
        }),

        HasSubmenu: function () {
            return _menuItems().length > 0;
        },

        //open : function(path)
        //{
        //    if (path.length == 1) {
        //        ko.utils.arrayForEach(_menuItems(), function (it) {
        //            if (it.Name == path[0])
        //                it.Click();
        //        });
        //    }
        //},

        Click: function () {

            console.log('click', _name);

            if (_menuItems().length > 0)
                return;

            if (typeof (opts.Action) == 'string')
                window.location = opts.Action;
            else if (typeof (opts.Action) == 'function') {
                opts.Action();
            }
        }
    };
};



//View Definition Module
//=====================================

NbuLib.ViewMgr = function () {

    var self = this;

    this.Loaded = ko.observable(false);
    this.Definitions = ko.observableArray([]);
    this.DefinitionsSorted = ko.computed(function () {
        var res = self.Definitions().slice().sort(function (a, b) { return a.Name.toLowerCase() > b.Name.toLowerCase() ? 1 : -1; });
        console.log('sorted def', res);
        return res;
    });

    $.ajax({
        type: 'GET',
        url: '/api/ViewDef/All',
        success: function (defs) {
            var es = new NbuLib.Core.EntityStore();
            $(defs).each(function () {
                var def = this;
                if (def.Entity)
                    es.getDefinition(def.Entity, function (eDef) {
                        self.Definitions.push(new NbuLib.ViewDefinition(def, eDef));
                    });
                else
                    self.Definitions.push(new NbuLib.ViewDefinition(def, null));
            });
            self.Loaded(true);
        }
    });
};

NbuLib.ViewMgr.prototype.save = function (viewDef) {
    console.log(viewDef);

    var view = {
        Name: viewDef.Name,
        Label: viewDef.Label,
        Entity: viewDef.Entity,
        Type: viewDef.Type,
        Fields: viewDef.Fields(),
        Filters: viewDef.Filters ? viewDef.Filters() : null
    };

    console.log('save view', view);

    client.ViewStore.saveView(view, function (res) {
        if (res.success)
            client.showSuccess('View definition saved successfully.');
    });
}

NbuLib.ViewMgr.prototype.addField2View = function (viewDef) {

    client.showInDialog(new NbuLib.ViewDefField(viewDef), {
        title: 'Add Field', templ: 'ViewMgr_AddViewField',
        act: {
            title: 'Add',
            callback: function (vm) {
                console.log(vm);
                viewDef.Fields.push(vm.getField());
                client.closeDialog();
            }
        }
    });
}

NbuLib.ViewMgr.prototype.addField2Form = function (viewDef) {

    client.showInDialog(new NbuLib.FormDefField(viewDef), {
        title: 'Add Field', templ: 'ViewMgr_AddFormField',
        act: {
            title: 'Add',
            callback: function (vm) {
                console.log(vm);
                viewDef.Fields.push(vm.getField());
                client.closeDialog();
            }
        }
    });
}
NbuLib.ViewMgr.prototype.addRelField2Form = function (viewDef) {

    client.showInDialog(new NbuLib.FormDefRel(viewDef), {
        title: 'Add Field', templ: 'ViewMgr_AddFormRel',
        act: {
            title: 'Add',
            callback: function (vm) {
                console.log(vm);
                viewDef.Fields.push(vm.getField());
                client.closeDialog();
            }
        }
    });
}

NbuLib.ViewMgr.prototype.addRelField2View = function (viewDef) {

    client.showInDialog(new NbuLib.ViewDefRelation(viewDef), {
        title: 'Add Relation Field', templ: 'ViewMgr_AddRelField',
        act: {
            title: 'Add',
            callback: function (vm) {
                console.log(vm);
                viewDef.Fields.push(vm.getField());
                client.closeDialog();
            }
        }
    });
}

NbuLib.ViewMgr.prototype.displayFormType = function (id) {
    if (id == 0)
        return 'Textbox';
}

NbuLib.ViewDefField = function (viewDef) {
    var self = this;

    this.Fields = [];
    this.Selected = ko.observable();
    this.SelectedType = ko.observable();
    this.FieldTypes = ko.observableArray([]);
    this.Result = {
        Property: ko.computed(function () { return !!self.Selected() ? self.Selected().Name : ''; }),
        Order: ko.observable(),
        Label: ko.observable(),
        Type: ko.computed(function () { return !!self.SelectedType() ? self.SelectedType().Value : null; }),
        Length: ko.observable(128),
        Multiline: ko.observable(false)
    };

    this.getField = function () {
        var field = {
            Property: self.Result.Property(),
            Order: self.Result.Order(),
            Role: null,
            Label: self.Result.Label(),
            Type: self.Result.Type()
        }

        if (field.Type == 0) //textfield
        {
            field["Length"] = self.Result.Length();
            field["Multiline"] = self.Result.Multiline();
        }
        else if (field.Type == 3) //enumfield
        {
            var prop = ko.utils.arrayFirst(viewDef.EntityDefinition.Properties, function (x) { return x.Name == field.Property; });
            field['EnumClass'] = prop.EnumClass;
        }

        return field;
    }

    this.Selected.subscribe(function () {
        self.FieldTypes.removeAll();
        if (!self.Selected())
            return;

        if (self.Selected().Type == 0) //string
        {
            self.FieldTypes.push({ Name: 'Textfield', Value: 0 });
            self.Result.Length(self.Selected().Length);
        }
        else if (self.Selected().Type == 1) //number
            self.FieldTypes.push({ Name: 'Numberfield', Value: 1 });
        else if (self.Selected().Type == 3) //boolean
            self.FieldTypes.push({ Name: 'Checkfield', Value: 4 });
        else if (self.Selected().Type == 4) //enum
            self.FieldTypes.push({ Name: 'Enumfield', Value: 3 });

    });

    //this.FieldTypes = ko.computed(function () {
    //    if (!self.Selected())
    //        return [];
    //    if (self.Selected().ValueTypeString == 'System.String')
    //        return [{Name: 'TextBox', Value: 1}];
    //});



    $(viewDef.EntityDefinition.Properties).each(function () {
        var prop = this;
        if (!ko.utils.arrayFirst(viewDef.Fields(), function (x) { return x.Property == prop.Name; }))
            self.Fields.push(this);
    });
}

NbuLib.ViewDefRelation = function (viewDef) {
    var self = this;

    this.Relations = ko.observableArray([]);
    this.Selected = ko.observable();
    this.SelectedField = ko.observable();
    this.SelectedType = ko.observable();
    this.FieldTypes = ko.observableArray([]);
    this.Result = {
        Role: ko.computed(function () { return !!self.Selected() ? self.Selected().Role : ''; }),
        Entity: ko.computed(function () { return !!self.Selected() ? self.Selected().Entity : ''; }),
        Property: ko.computed(function () { return !!self.SelectedField() ? self.SelectedField().Name : ''; }),
        Order: ko.observable(),
        Label: ko.observable(),
        Type: ko.computed(function () { return !!self.SelectedType() ? self.SelectedType().Value : null; }),
        Length: ko.observable(128),
        Multiline: ko.observable(false)
    };

    this.getField = function () {
        var field = {
            Role: self.Result.Role(),
            Entity: self.Result.Entity(),
            Property: self.Result.Property(),
            Order: self.Result.Order(),
            Label: self.Result.Label(),
            Type: self.Result.Type()
        }

        if (field.Type == 0) //textfield
        {
            field["Length"] = self.Result.Length();
            field["Multiline"] = self.Result.Multiline();
        }

        console.log('getField', field);

        return field;
    }

    this.SelectedField.subscribe(function () {
        self.FieldTypes.removeAll();
        if (!self.SelectedField())
            return;

        if (self.SelectedField().Type == 0) //string
        {
            self.FieldTypes.push({ Name: 'Textfield', Value: 0 });
            self.Result.Length(self.SelectedField().Length);
        }

        if (self.SelectedField().Type == 3) //boolean
            self.FieldTypes.push({ Name: 'Textfield', Value: 0 });
        else if (self.SelectedField().Type == 4) //enum
            self.FieldTypes.push({ Name: 'Enumfield', Value: 3 });

    });

    //this.FieldTypes = ko.computed(function () {
    //    if (!self.Selected())
    //        return [];
    //    if (self.Selected().ValueTypeString == 'System.String')
    //        return [{Name: 'TextBox', Value: 1}];
    //});


    var es = new NbuLib.Core.EntityStore();
    $(viewDef.EntityDefinition.Relations).each(function () {
        var rel = {
            Role: this.Role,
            Entity: viewDef.EntityDefinition.Name == this.LeftEntity ? this.RightEntity : this.LeftEntity,

        };

        rel.Name = rel.Entity + '_' + rel.Role;

        if (!ko.utils.arrayFirst(viewDef.Fields(), function (x) { return x.Entity == rel.Entity && x.Role == rel.Role; })) {
            es.getDefinition(rel.Entity, function (def) {
                rel.Fields = def.Properties;
                self.Relations.push(rel);
            });
        }
    });
}

NbuLib.FormDefField = function (viewDef) {
    var self = this;

    this.Fields = [];
    this.Selected = ko.observable();
    this.SelectedType = ko.observable();
    this.FieldTypes = ko.observableArray([]);
    this.Result = {
        Property: ko.computed(function () { return !!self.Selected() ? self.Selected().Name : ''; }),
        Order: ko.observable(),
        Required: ko.observable(),
        Label: ko.observable(),
        Type: ko.computed(function () { return !!self.SelectedType() ? self.SelectedType().Value : null; }),

        //Textbox
        MinLength: ko.observable(),
        MaxLength: ko.observable(),
        Multiline: ko.observable(false)
    };

    this.getField = function () {
        var field = {
            Property: self.Result.Property(),
            Order: self.Result.Order(),
            Required: self.Result.Required(),
            Label: self.Result.Label(),
            Type: self.Result.Type()
        }

        if (field.Type == 0) //textbox
        {
            field["MinLength"] = self.Result.MinLength();
            field["MaxLength"] = self.Result.MaxLength();
            field["Multiline"] = self.Result.Multiline();
        }
        else if (field.Type == 3) //enumlist
        {
            var prop = ko.utils.arrayFirst(viewDef.EntityDefinition.Properties, function (x) { return x.Name == field.Property; });
            field['EnumClass'] = prop.EnumClass;
        }

        return field;
    }

    this.Selected.subscribe(function () {

        self.FieldTypes.removeAll();
        self.SelectedType(null);
        if (!self.Selected())
            return;

        self.Result.Required(!self.Selected().Nullable);

        if (self.Selected().Type == 0) //string
        {
            self.FieldTypes.push({ Name: 'Textbox', Value: 0 });
            self.Result.MaxLength(self.Selected().Length);
        }
        else if (self.Selected().Type == 1) //number
            self.FieldTypes.push({ Name: 'Numberfield', Value: 1 });
            //if (self.Selected().Type == 3) //boolean
            //    self.FieldTypes.push({ Name: 'Textfield', Value: 0 });
        else if (self.Selected().Type == 3) //boolean
            self.FieldTypes.push({ Name: 'Checkbox', Value: 5 });
        else if (self.Selected().Type == 4) //enum
            self.FieldTypes.push({ Name: 'Enumlist', Value: 3 });
        else if (self.Selected().Type == 5) //date
            self.FieldTypes.push({ Name: 'Datepicker', Value: 2 });

    });

    //this.FieldTypes = ko.computed(function () {
    //    if (!self.Selected())
    //        return [];
    //    if (self.Selected().ValueTypeString == 'System.String')
    //        return [{Name: 'TextBox', Value: 1}];
    //});



    $(viewDef.EntityDefinition.Properties).each(function () {
        var prop = this;
        if (!ko.utils.arrayFirst(viewDef.Fields(), function (x) { return x.Property == prop.Name; }))
            self.Fields.push(this);
    });
}

NbuLib.FormDefRel = function (viewDef) {
    var self = this;

    this.Fields = [];
    this.Selected = ko.observable();
    this.SelectedType = ko.observable();
    this.FieldTypes = ko.observableArray([{ Name: 'Selectlist', Value: 4 }]);
    this.Relations = ko.observableArray([]);

    this.RelProperties = ko.computed(function () {
        if (!self.Selected())
            return [];
        var es1 = new NbuLib.Core.EntityStore();
        var ed = es1.getDefinition(self.Selected().Entity);
        return ed.Properties;
    });

    this.Result = {
        Property: ko.computed(function () { return !!self.Selected() ? self.Selected().Name : ''; }),
        Entity: ko.computed(function () { return !!self.Selected() ? self.Selected().Entity : ''; }),
        Role: ko.computed(function () { return !!self.Selected() ? self.Selected().Role : ''; }),
        Formula: ko.observable(),
        Order: ko.observable(),
        Required: ko.observable(),
        Label: ko.observable(),
        Type: ko.computed(function () { return !!self.SelectedType() ? self.SelectedType().Value : null; }),
        Multiple: ko.computed(function () {
            if (!self.Selected())
                return '';

            if (self.Selected().LeftEntity == viewDef.EntityDefinition.Name)
                return self.Selected().Type == 1 // one to many
                    || self.Selected().Type == 3; //many to many
            else
                return self.Selected().Type == 2 // many to one
                    || self.Selected().Type == 3; //many to many
        })
    };

    this.getField = function () {
        var field = {
            Property: self.Result.Property(),
            Entity: self.Result.Entity(),
            Role: self.Result.Role(),
            Formula: self.Result.Formula(),
            Order: self.Result.Order(),
            Required: self.Result.Required(),
            Multiple: self.Result.Multiple(),

            Label: self.Result.Label(),
            Type: self.Result.Type()
        }

        if (field.Type == 0) //textbox
        {
            field["MinLength"] = self.Result.MinLength();
            field["MaxLength"] = self.Result.MaxLength();
            field["Multiline"] = self.Result.Multiline();
        }
        else if (field.Type == 3) //enumlist
        {
            var prop = ko.utils.arrayFirst(viewDef.EntityDefinition.Properties, function (x) { return x.Name == field.Property; });
            field['EnumClass'] = prop.EnumClass;
        }

        return field;
    }

    this.getHint = function () {

        var opts = [];
        var es1 = new NbuLib.Core.EntityStore();
        var ed = es1.getDefinition(self.Selected().Entity);
        $(ed.Properties).each(function () {
            opts.push(this.Name);
        });
        return {
            title: 'Choose what properties to display',
            content: '<p>You can use the following properties of the ' + self.Selected().Entity + ' entity: </p>' + opts.join(', '),
            html: true,
            trigger : 'manual'
        };

    }

    var es = new NbuLib.Core.EntityStore();
    $(viewDef.EntityDefinition.Relations).each(function () {
        var rel = {
            Role: this.Role,
            Type: this.Type,
            Entity: viewDef.EntityDefinition.Name == this.LeftEntity ? this.RightEntity : this.LeftEntity,

        };

        rel.Name = rel.Entity + '_' + rel.Role;

        if (!ko.utils.arrayFirst(viewDef.Fields(), function (x) { return x.Entity == rel.Entity && x.Role == rel.Role; })) {
            self.Relations.push(rel);
        }
    });
}

NbuLib.ViewDefinition = function (data, entityDef) {
    this.Fields = ko.observableArray(data.Fields || []);
    this.Label = data.Label;
    this.Type = data.Type;
    this.Entity = data.Entity;
    this.EntityDefinition = entityDef;
    this.Name = data.Name;
    var self = this;
    this.remove = function (field) {
        self.Fields.remove(field);
        //ko.utils.arrayRemoveItem(self.Fields(), field);
    }
};

(function () {
    client.hookMenuInit(function (menu) {
        menu.AddItem(new MenuViewModel({
            Name: 'View',
            Order: 2,
            Action: function () {
                client.showInViewport(new NbuLib.ViewMgr(), { templ: 'ViewMgr' });
            }
        }));


        //for (var x in menu.MenuItems()) {
        //    if (menu.MenuItems()[x].Name == 'Profile')
        //        menu.MenuItems()[x].MenuItems.push({Name : 'Change Password', Order : 1});
        //}
    });
})();