﻿NbuLib.Core.EntityVM = function (viewDefinition) {

    this.Entity = {};
    this._relCollections = {};
    this.Loaded = ko.observable(false);
    this.LoadedDef = ko.observable(false);
    this.Definition = null;

    var self = this;

    if (viewDefinition && typeof (viewDefinition) == 'object') {
        this.Definition = viewDefinition;
        this.LoadedDef(true);
    }
    else {
        self.Definition = client.ViewStore.getView(viewDefinition || self.ViewName);
        self.LoadedDef(true);

        //$.get('/api/ViewDef/Get', { viewName: viewDefinition || self.ViewName }, function (def) {
        //    self.Definition = def;
        //    self.LoadedDef(true);
        //});
    }



    $(self.Definition.Fields).each(function () {
        if (this.Entity) {
            self.Entity[this.Entity + '_' + this.Role] = {};
        }
        else
            self.Entity[this.Property] = ko.protectedObservable();
    });
}

NbuLib.Core.EntityVM.prototype.load = function (id, entity) {
    var self = this;

    var includes = [];

    $(self.Definition.Fields).each(function () {
        if (this.Entity) {
            includes.push({ EntityName: this.Entity, Role: this.Role });
        }
    });

    client.get(id, entity, includes, function (data) {
        self.loadData(data);
    });
}

NbuLib.Core.EntityVM.prototype.getChanges = function () {
    var self = this;

    var changes = [];

    $.each(self.Entity, function (k, v) {
        if (k == 'EntityName')
            return;

        if (v && v.hasChanges && v.hasChanges())
            changes.push({ Property: k, Value: v });
        else
            console.log(k, v);
    });

    return changes;
}

NbuLib.Core.EntityVM.prototype.save = function (callback) {
    var self = this;

    var update = { Id: self.Entity.Id ? self.Entity.Id() : null, Entity: self.EntityName, Props: {}, Rels: [] };
    var changes = self.getChanges();
    $(changes).each(function () {
        this.Value.commit();
        update.Props[this.Property] = this.Value;
    });

    $(self.Definition.Fields).each(function () {
        if (!this.Entity) {
            ;//update.Props[this.Property] = self.Entity[this.Property]();
        }
        else {
            //TODO
            var field = this;
            var key = field.Entity + '_' + field.Role;
            $(self.Entity[key]).each(function () {
                var item = this;
                if (!!self._relCollections[key] && !!ko.utils.arrayFirst(self._relCollections[key], function (x) { return x.Id == item.Id }))
                    return;
                else
                    update.Rels.push({ Entity: field.Entity, Id: item.Id, Role: field.Role, Op: 1 }); //attach
            });
            if (!!self._relCollections[key] && self._relCollections[key].length)
                $(self._relCollections[key]).each(function () {
                    var olditem = this;
                    if (!!ko.utils.arrayFirst(self.Entity[key], function (x) { return x.Id == olditem.Id }))
                        return;
                    else
                        update.Rels.push({ Entity: field.Entity, Id: olditem.Id, Role: field.Role, Op: 2 }); //detach
                });
        }
    });

    client.update(update, callback);
}

NbuLib.Core.EntityVM.prototype.loadData = function (data) {
    var self = this;

    $.each(data, function (k, v) {
        if (typeof (v) == "function")
            self.Entity[k] = ko.protectedObservable(v())
        else if (v === null)
            self.Entity[k] = ko.protectedObservable();
        else if (typeof (v) == "object")
            self.Entity[k] = v;
        else if (v || v === 0)
            self.Entity[k] = ko.protectedObservable(v);
        else
            self.Entity[k] = ko.protectedObservable("");
    });
    self.Loaded(true);
}

NbuLib.Core.EntityVM.prototype.addRelation = function (rel, item) {
    var self = this;
    var key = rel.Entity + "_" + rel.Role;

    //save initial state
    if (!self._relCollections[key]) {
        self._relCollections[key] = [];
        ko.utils.arrayPushAll(self._relCollections[key], self.Entity[key]);
    }

    //TODO
    if (!self.Entity[key].push) {
        self.Entity[key] = [];
    }

    self.Entity[key].push(item);

    console.log('addRelation(' + key + ')', item);
}

NbuLib.Core.EntityVM.prototype.removeRelation = function (rel, item) {
    var self = this;
    var key = rel.Entity + "_" + rel.Role;

    //save initial state
    if (!self._relCollections[key]) {
        self._relCollections[key] = [];
        ko.utils.arrayPushAll(self._relCollections[key], self.Entity[key]);
    }

    var deletedItem = ko.utils.arrayFirst(self.Entity[key], function (x) { return x.Id == item.Id; });
    ko.utils.arrayRemoveItem(self.Entity[key], deletedItem);

    console.log('removeRelation(' + key + ')', item);
}

NbuLib.Core.EntityVM.prototype.EntityName = "";