﻿NbuLib.Core.ListVM = function () {
    this.Items = ko.observableArray([]);
    this.Selected = ko.observable();
    this.Loaded = ko.observable(false);
    this.LoadedDef = ko.observable(false);

    this.Actions = [];
    this.DefaultAction = null;
    this.GlobalActions = [];
    this.Definition = null;

    var self = this;

    self.sortBy = function (field) {

        self.clearItems();
        var sort = null;
        $(self.eq.SortBy).each(function () {
            if (this.Property == field.Property)
                sort = this;
        });
        if (sort) {
            if (!sort.Descending)
                sort.Descending = !sort.Descending;
            else
                ko.utils.arrayRemoveItem(self.eq.SortBy, sort);
        }
        else
            self.eq.SortBy.push(NbuLib.QueryBuilder.makeSort(field.Property, false, field.Role));

        self.reload();
    };

    self.Definition = client.ViewStore.getView(this.ViewName);
    self.LoadedDef(true);

    //$.get('/api/ViewDef/Get', { viewName: this.ViewName }, function (def) {
    //    self.Definition = def;
    //    self.LoadedDef(true);
    //});
}

NbuLib.Core.ListVM.prototype.ViewName = "";

NbuLib.Core.ListVM.prototype.AjaxSource = { Url: null, Args: null, Callback: null };

NbuLib.Core.ListVM.prototype.clearItems = function () {
    this.Loaded(false);
    this.Items.removeAll();
}

NbuLib.Core.ListVM.prototype.loadItems = function (data) {
    var self = this;

    $(data).each(function () {
        var item = this;
        $.each(item, function (k, v) {
            if (typeof (v) == "function" || typeof (v) == "object")
                item[k] = v;
            else if (v || v === 0)
                item[k] = ko.observable(v);
            else
                item[k] = ko.observable("");
        });
        self.Items.push(item);
    });
    self.Loaded(true);
}

NbuLib.Core.ListVM.prototype.loadAjax = function (url, args, callback) {
    var self = this;

    if (!callback)
        callback = function (items) {
            self.loadItems(items);
        };

    //if (!args)
    //    args = {};

    //$.extend(args, { ts: new Date().getTime() });

    $.post(url, args, callback);
    self.AjaxSource.Url = url;
    self.AjaxSource.Args = args;
    self.AjaxSource.Callback = callback;

    self.Loaded(false);
    client.showLoader(self);
}

NbuLib.Core.ListVM.prototype.load = function (query) {
    var self = this;

    var include = [];

    $(self.Definition.Fields).each(function () {
        if (this.Entity) {
            include.push({ EntityName: this.Entity, Role: this.Role });
        }
    });

    self.eq = { Entity: query.Entity, Filter: query.Filter || [], RelFilter: query.RelFilter || [], SortBy: query.SortBy || [], Includes: include };

    client.search(self.eq, function (items) {
        self.loadItems(items);
    });


    self.Loaded(false);
    client.showLoader(self);
}

NbuLib.Core.ListVM.prototype.reload = function () {
    this.clearItems();
    this.load(this.eq);
    //this.loadAjax(this.AjaxSource.Url, this.AjaxSource.Args, this.AjaxSource.Callback);

}

NbuLib.Core.ListVM.prototype.getSortBy = function (field) {
    var self = this;

    for (var x in self.eq.SortBy) {
        if (self.eq.SortBy[x].Property == field.Property)
            return self.eq.SortBy[x].Descending;
    }
}

NbuLib.Core.ListVM.prototype.filterBy = function (field, values) {
    console.log(values);
    var self = this;
    var eq = self.eq;
    var filter = self.getFilterBy(field);
    if (filter) {
        if (field.Entity) {
            if (!values)
                ko.utils.arrayRemoveItem(eq.RelFilter, filter);
            else
                filter.PropertyRules[0].Values = values;
        }
        else {
            if (!values)
                ko.utils.arrayRemoveItem(eq.Filter, filter);
            else
                filter.Values = values;
        }
    }
    else {
        var op = "is";
        if (field.Type === 0)
            op = "sw"
        if (field.Entity)
            eq.RelFilter.push(NbuLib.QueryBuilder.makeRelFilter(field.Entity, field.Role, "Id", "is", values));
        else
            eq.Filter.push(NbuLib.QueryBuilder.makeFilter(field.Property, op, values));
    }
    //self.reload();
}

NbuLib.Core.ListVM.prototype.getFilterBy = function (field) {
    for (var x in this.eq.Filter) {
        if (this.eq.Filter[x].Property == field.Property)
            return this.eq.Filter[x];
    }
    for (var x in this.eq.RelFilter) {
        if (this.eq.RelFilter[x].EntityName == field.Entity && this.eq.RelFilter[x].Role == field.Role)
            return this.eq.RelFilter[x];
    }
}

NbuLib.Core.ListVM.prototype.applyFilters = function () {
    this.reload();
}


//TODO MOVE FROM HERE
//========================================

NbuLib.MyMagazines = {};

NbuLib.MyMagazines.MagazinesVM = function () {
    NbuLib.MyMagazines.MagazinesVM.base.call(this);

    var self = this;

    //$.get('/api/AskTheLib/GetAllQueries', { ts: new Date().getTime(), processedByMe: processedByMe }, function (data) {
    //    self.loadItems(data);
    //});

    //self.loadAjax('/api/AskTheLib/GetAllQueries', { processedByMe: processedByMe });

    self.load({ Entity: 'Magazine' });

    //self.Actions.push({ Label: 'Process', Callback: self.edit, Icon: 'icon-chevron-right' });
    //self.DefaultAction = { Label: 'Process', Callback: self.process, Icon: 'icon-chevron-right' };
    //self.Actions.push({ Label: 'Open', Callback: self.open, Icon: 'icon-list-alt' });
    //self.Actions.push({ Label: 'Edit', Callback: self.edit, Icon: 'icon-pencil' });
    //    self.Actions.push({ Label: 'Delete', Callback: self.remove, Icon: 'icon-trash' });

    self.GlobalActions.push({ Label: 'New Magazine', Callback: self.create });
}
NbuLib.extend(NbuLib.MyMagazines.MagazinesVM, NbuLib.Core.ListVM);

NbuLib.MyMagazines.MagazinesVM.prototype.ViewName = "MyMagazines_Librarian_MagazinesGrid";

NbuLib.MyMagazines.MagazinesVM.prototype.create = function () {
    var self = this;

    var vm = new NbuLib.Core.EntityVM("MyMagazines_Librarian_MagazinesNewForm");
    vm.EntityName = 'Magazine';
    vm.Loaded(true);
    client.showInDialog(vm, {
        templ: 'GeneralForm', title: 'New Magazine', act: {
            title: 'Create', callback: function (updates) {
                updates.save(function (response) {
                    //console.log(response);
                    if (response.success) {
                        client.closeDialog();

                        //self.updates.Entity.Id(resp.sync.Id);
                        //self.Items.push(updates.Entity);

                        self.reload();

                        client.showSuccess("Saved successfully.");
                    }
                });
            }
        }
    });
}